unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls,
  OpenGLContext;

type

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    OpenGLControl1: TOpenGLControl;
    Panel1: TPanel;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure OpenGLControl1Paint(Sender: TObject);
    procedure Panel1Paint(Sender: TObject);
  private
    FBitmap: TBitmap;

  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

uses
  FPImage, FPImgCanv, IntfGraphics,
  gl, glu;

{ TForm1 }

const
  N = 10000;

procedure TForm1.Button1Click(Sender: TObject);
var
  t: TDateTime;
  seconds: Double;
  i: Integer;
  P1, P2, P3: TPoint;
  w, h: Integer;
begin
  OpenGLControl1.Hide;
  Panel1.Show;

  t := Now;

  if FBitmap = nil then
    FBitmap := TBitmap.Create;
  w := Panel1.ClientWidth;
  h := Panel1.ClientHeight;
  FBitmap.SetSize(w, h);

  FBitmap.Canvas.Brush.Color := clWhite;
  FBitmap.Canvas.FillRect(0, 0, w, h);

  for i := 1 to N do
  begin
    P1 := Point(Random(w), Random(h));
    P2 := Point(Random(w), Random(h));
    P3 := Point(Random(w), Random(h));
    FBitmap.Canvas.Brush.Color := random($FFFFFF);
    FBitmap.Canvas.Polygon([P1, P2, P3]);
  end;

  Panel1.Invalidate;

  t := Now - t;
  seconds := t * (24*60*60);
  Caption := Format('%.3f sec --> %.0n triangles/sec', [seconds, N/seconds]);
end;

procedure TForm1.Button2Click(Sender: TObject);
var
  t: TDateTime;
  seconds: Double;
  i: Integer;
  img: TLazIntfImage;
  w, h: Integer;
  P1, P2, P3: TPoint;
  clr: TFPColor;
  lCanvas: TFPImageCanvas;
begin
  OpenGLControl1.Hide;
  Panel1.Show;

  t := Now();

  if FBitmap = nil then
    FBitmap := TBitmap.Create;
  w := Panel1.ClientWidth;
  h := Panel1.ClientHeight;
  FBitmap.SetSize(w, h);

  img := FBitmap.CreateIntfImage;
  try
    lCanvas := TFPImageCanvas.Create(img);
    lCanvas.Brush.FPColor := colWhite;
    lCanvas.FillRect(0, 0, w, h);
    for i := 1 to N do
    begin
      P1 := Point(Random(w), Random(h));
      P2 := Point(Random(w), Random(h));
      P3 := Point(Random(w), Random(h));
//      clr := TColorToFPColor(Random($FFFFFF));

      clr.Red := random(65535);
      clr.Green := random(65535);
      clr.Blue := random(65535);
      clr.Alpha := 65535;

      lCanvas.Brush.FPColor := clr;
      lCanvas.Polygon([P1, P2, P3]);
    end;
    FBitmap.LoadFromIntfImage(img);
  finally
    img.Free;
  end;

  Panel1.Invalidate;

  t := Now - t;
  seconds := t * (24*60*60);
  Caption := Format('%.3f sec --> %.0n triangles/sec', [seconds, N/seconds]);
end;

procedure TForm1.Button3Click(Sender: TObject);
var
  t: TDateTime;
  seconds: double;
begin
  Panel1.Hide;
  OpenGLControl1.Show;
  t := Now;
  OpenGLControl1.Refresh;
  t := Now - t;
  seconds := t * 24*60*60;
  Caption := Format('%.3f sec --> %.0n triangles/sec', [seconds, N/seconds]);
end;

procedure TForm1.Button4Click(Sender: TObject);
var
  t: TDateTime;
  seconds: double;
begin
  Panel1.Show;
  OpenGLControl1.Hide;
  FreeAndNil(FBitmap);
  t := Now;
  Panel1.Refresh;
  t := Now - t;
  seconds := t * 24*60*60;
  Caption := Format('%.3f sec --> %.0n triangles/sec', [seconds, N/seconds]);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  FBitmap.Free;
end;

procedure TForm1.OpenGLControl1Paint(Sender: TObject);
var
  i: Integer;
  w, h: Integer;
begin
  w := OpenGLControl1.Width;
  h := OpenGLControl1.Height;

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity;
  glViewport(0,0,w, h);
  gluOrtho2D(0, 1, 0, 1);

  glMatrixMode(GL_MODELVIEW);
  glClearColor(1.0, 1.0, 1.0, 1.0);
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
  glLoadIdentity;
  for i := 1 to N do begin
    glBegin(GL_TRIANGLES);
      glColor3f(Random, Random, Random);
      glVertex2f(Random, Random);       // P1
      glVertex2f(Random, Random);       // P2
      glVertex2f(Random, Random);       // P3
    glEnd;
  end;
  OpenGLControl1.SwapBuffers;
end;

procedure TForm1.Panel1Paint(Sender: TObject);
var
  w, h, i: Integer;
  P1, P2, P3: TPoint;
begin
  if FBitmap <> nil then
    Panel1.Canvas.Draw(0, 0, FBitmap)
  else
  begin
    w := Panel1.ClientWidth;
    h := Panel1.ClientHeight;
    Panel1.Canvas.Brush.Color := clWhite;
    Panel1.Canvas.FillRect(0, 0, w, h);
    for i := 1 to N do
    begin
      P1 := Point(Random(w), Random(h));
      P2 := Point(Random(w), Random(h));
      P3 := Point(Random(w), Random(h));
      Panel1.Canvas.Brush.Color := random($FFFFFF);
      Panel1.Canvas.Polygon([P1, P2, P3]);
    end;
  end;
end;

end.

